# FiloBlu Order Additional Data # 
 
Save additional data on order place. Data is serialized and put in a separate data structure.

## Notes on Extension Attributes

Magento 2 introduced Extension Attributes, a way to extend original interfaces without changing them.
This feature has been adopted in this module to add data to orders and order items.

As of November 2016 there is very little documentation on how Extension Attributes actually work and how to use them. 
The original [Magento Documentation](http://devdocs.magento.com/guides/v2.0/extension-dev-guide/attributes.html#search) is very theoric 
and somehow confusing especially in the part concerning joins. After some research, we found out a 
[Magento Stack Exchange thread](http://magento.stackexchange.com/questions/121100/how-to-add-extension-attribute-magento-2) explaining 
that not all the interfaces can be "joined": it depends on the injection of JoinProcessorInterface. 
In our case, orders and order items cannot be extended with joins.

The steps to create a new extension attribute:
1. create the file *extension_attributes.xml* defining which interface you are extending and the type of attribute. The attribute can be 
either one of core data types (int, float[], string, array) or an interface (though also a class can be used, this should be avoided due to API related reasons). 
We explored the interface solution but finally we chose an *array* attribute for the sake of simplicity and considering some api-related aspects. 
2. some methods to read/write data are needed. We adopted *plugin pattern* related to Order Repository and Order Item Repository interfaces.
3. compile. Two methods are added to *var/generation/Magento/Sales/Api/Data/OrderExtension.php*

## Extension attributes and APIs

The extension attributes are automatically added to the output of APIs. In APIs complex datas are converted to XML/JSON through 
\Magento\Framework\Webapi\ServiceOutputProcessor::process but the actual core funtions deputed to conversions are:
* \Magento\Framework\Reflection\ExtensionAttributesProcessor::buildOutputDataArray
* \Magento\Framework\Reflection\DataObjectProcessor::buildOutputDataArray
* \Magento\Framework\Reflection\CustomAttributesProcessor::buildOutputDataArray

If the extension attributes are interfaces, the conversion creates the output by taking all the methods of interface. Please note that in this context
the comments before functions are parsed and the @return type is mandatory.

Special Attention has to be given to the case of *array data*. Processors (esp. DataObjectProcessor, rows 108-117) strip out array keys for the first level 
and leave the keys of nested arrays untouched. This is the reason why our Additional Data array is incapsulated in another array. The same behavior 
showed up even using interface and a getDataArray funciton.